from plivo.base import (ListResponseObject, PlivoResource,
                        PlivoResourceInterface)
from plivo.utils import to_param_dict
from plivo.utils.validators import *


class MaskingSession(PlivoResource):
    _name = 'MaskingSession'
    _identifier_string = 'session_uuid'

    def create_masking_session(self):
        return self.client.calls.create_masking_session(self.id,
                                                        **to_param_dict(self.create_masking_session(), locals()))

    def delete_masking_session(self):
        return self.client.calls.delete_masking_session(self.id,
                                                        **to_param_dict(self.delete_masking_session(), locals()))

    def get_masking_session(self):
        return self.client.calls.get_masking_session(self.id,
                                                     **to_param_dict(self.get_masking_session(), locals()))

    def update_masking_session(self):
        return self.client.calls.update_masking_session(self.id,
                                                        **to_param_dict(self.update_masking_session(), locals()))

    def list_masking_session(self):
        return self.client.calls.list_masking_session(self.id,
                                                      **to_param_dict(self.list_masking_session(), locals()))


class MaskingSessions(PlivoResourceInterface):
    _resource_type = MaskingSession

    @validate_args(
        callback_url=[optional(is_url())],
        recording_callback_url=[optional(is_url())],
        first_party_play_url=[optional(is_url())],
        second_party_play_url=[optional(is_url())],
        pin_prompt_play=[optional(is_url())],
        incorrect_pin_play=[optional(is_url())],
        unknown_caller_play=[optional(is_url())],
    )
    def create_masking_session(self,
                               first_party=None,
                               second_party=None,
                               session_expiry=None,
                               call_time_limit=None,
                               record=None,
                               record_file_format=None,
                               recording_callback_url=None,
                               initiate_call_to_first_party=None,
                               callback_url=None,
                               callback_method=None,
                               ring_timeout=None,
                               first_party_play_url=None,
                               second_party_play_url=None,
                               recording_callback_method=None,
                               is_pin_authentication_required=None,
                               generate_pin=None,
                               generate_pin_length=None,
                               first_party_pin=None,
                               second_party_pin=None,
                               pin_prompt_play=None,
                               pin_retry=None,
                               pin_retry_wait=None,
                               incorrect_pin_play=None,
                               unknown_caller_play=None,
                               subaccount=None,
                               geomatch=None
                               ):
        return self.client.request('POST', ('Masking', 'Session',),
                                   to_param_dict(self.create_masking_session, locals()), is_voice_request=True)

    def delete_masking_session(self,
                               session_uuid
                               ):
        return self.client.request('DELETE', ('Masking', 'Session', session_uuid),
                                   to_param_dict(self.delete_masking_session, locals()), is_voice_request=True)

    def get_masking_session(self,
                            session_uuid
                            ):
        return self.client.request('GET', ('Masking', 'Session', session_uuid), is_voice_request=True)

    def update_masking_session(self,
                               session_uuid,
                               session_expiry=None,
                               call_time_limit=None,
                               record=None,
                               record_file_format=None,
                               recording_callback_url=None,
                               callback_url=None,
                               callback_method=None,
                               ring_timeout=None,
                               first_party_play_url=None,
                               second_party_play_url=None,
                               recording_callback_method=None,
                               subaccount=None,
                               geomatch=None
                               ):
        return self.client.request('POST', ('Masking', 'Session', session_uuid),
                                   to_param_dict(self.update_masking_session, locals()), is_voice_request=True)

    def list_masking_session(self,
                             first_party=None,
                             second_party=None,
                             virtual_number=None,
                             status=None,
                             created_time=None,
                             created_time__lt=None,
                             created_time__lte=None,
                             created_time__gt=None,
                             created_time__gte=None,
                             expiry_time=None,
                             expiry_time__lt=None,
                             expiry_time__lte=None,
                             expiry_time__gt=None,
                             expiry_time__gte=None,
                             duration=None,
                             duration__lt=None,
                             duration__lte=None,
                             duration__gt=None,
                             duration__gte=None,
                             limit=None,
                             offset=None,
                             subaccount=None
                             ):
        return self.client.request('GET', ('Masking', 'Session'), to_param_dict(self.list_masking_session, locals()),
                                   is_voice_request=True)



from plivo.utils.validators import validate_args, optional, of_type_exact, validate_list_items, validate_dict_items

class Header:
    @validate_args(
        type=[optional(of_type_exact(str, type(None)))],
        text=[optional(of_type_exact(str, type(None)))],
        media=[optional(of_type_exact(str, type(None)))]
    )
    def __init__(self, type=None, text=None, media=None):
        self.type = type
        self.text = text
        self.media = media

class Body:
    @validate_args(
        text=[optional(of_type_exact(str, type(None)))]
    )
    def __init__(self, text=None):
        self.text = text

class Footer:
    @validate_args(
        text=[optional(of_type_exact(str, type(None)))]
    )
    def __init__(self, text=None):
        self.text = text

class Row:
    @validate_args(
        id=[optional(of_type_exact(str, type(None)))],
        title=[optional(of_type_exact(str, type(None)))],
        description=[optional(of_type_exact(str, type(None)))]
    )
    def __init__(self, id=None, title=None, description=None):
        self.id = id
        self.title = title
        self.description = description

class Section:
    @validate_args(
        title=[optional(of_type_exact(str, type(None)))],
        rows=[optional(validate_list_items(Row))]
    )
    def __init__(self, title=None, rows=None):
        self.title = title
        self.rows = rows if rows is not None else []

class Btn:
    @validate_args(
        id=[optional(of_type_exact(str, type(None)))],
        title=[optional(of_type_exact(str, type(None)))],
        cta_url=[optional(of_type_exact(str, type(None)))]
    )
    def __init__(self, id=None, title=None, cta_url=None):
        self.id = id
        self.title = title
        self.cta_url = cta_url

class Action:
    @validate_args(
        buttons=[optional(validate_list_items(Btn))],
        sections=[optional(validate_list_items(Section))]
    )
    def __init__(self, buttons=None, sections=None):
        self.buttons = buttons if buttons is not None else []
        self.sections = sections if sections is not None else []

class Interactive:
    @validate_args(
        type=[optional(of_type_exact(str, type(None)))],
        header=[optional(validate_dict_items(Header))],
        body=[optional(validate_dict_items(Body))],
        footer=[optional(validate_dict_items(Footer))],
        action=[optional(validate_dict_items(Action))]
    )
    def __init__(self, type=None, header=None, body=None, footer=None, action=None):
        self.type = type
        # Assign directly the validated dictionary or default to None if not provided
        self.header = header
        self.body = body
        self.footer = footer
        self.action = action



from plivo.utils.validators import *
from plivo.xml import PlivoXMLElement, map_type


class BreakElement(PlivoXMLElement):
    _name = 'break'
    _nestable = []

    @property
    def strength(self):
        return self.__strength

    @strength.setter
    def strength(self, value):
        self.__strength = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_strength(self, value):
        self.strength = value
        return self

    @property
    def time(self):
        return self.__time

    @time.setter
    def time(self, value):
        self.__time = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_time(self, value):
        self.time = value
        return self

    def __init__(
        self,
        content=None,
        strength=None,
        time=None,
    ):

        super(BreakElement, self).__init__()
        self.content = content
        self.strength = strength
        self.time = time

    def to_dict(self):
        d = {
            'strength': self.strength,
            'time': self.time,
        }

        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }



# -*- coding: utf-8 -*-
from plivo.utils.validators import *

from ..base import ListResponseObject, PlivoResource, PlivoResourceInterface, ListSessionResponseObject
from ..exceptions import *
from ..utils import *


class Session(PlivoResource):
    _name = 'Session'
    _identifier_string = 'session_uuid'

    def delete(self):
        raise InvalidRequestError('Cannot delete a Session resource')

    def update(self):
        raise InvalidRequestError('Cannot update a Session resource')


class Sessions(PlivoResourceInterface):
    _resource_type = Session

    @validate_args(
        app_uuid=[optional(of_type(six.text_type))],
        otp=[optional(of_type(six.text_type))],
        recipient=[required(is_phonenumber())],
        channel=[optional(all_of(of_type(six.text_type), is_in(('sms', 'voice'))))],
        url=[optional(is_url())],
        method=[optional(of_type(six.text_type))])
    def create(self,
               app_uuid=None,
               otp=None,
               recipient=None,
               channel=None,
               url=None,
               method='POST'):
        if recipient is None:
            raise ValidationError('destination number is required')
        return self.client.request('POST', ('Verify', 'Session', ),
                                   to_param_dict(self.create, locals()))

    @validate_args(session_uuid=[of_type(six.text_type)])
    def get(self, session_uuid):
        return self.client.request(
            'GET', ('Verify', 'Session', session_uuid), response_type=Session)

    @validate_args(
        subaccount=[optional(is_subaccount())],
        status=[optional(is_in(('in-progress', 'expired', 'verified')))],
        session_time__gt=[optional(is_valid_date())],
        session_time__gte=[optional(is_valid_date())],
        session_time__lt=[optional(is_valid_date())],
        session_time__lte=[optional(is_valid_date())],
        session_time=[optional(is_valid_date())],
        country=[optional(of_type(six.text_type))],
        alias=[optional(of_type(six.text_type))],
        app_uuid=[optional(of_type(six.text_type))],
        recipient=[optional(of_type(six.text_type))],
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda limit: 0 < limit <= 20, '0 < limit <= 20')))
        ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list(self,
             subaccount=None,
             status=None,
             session_time__gt=None,
             session_time__gte=None,
             session_time__lt=None,
             session_time__lte=None,
             session_time=None,
             country=None,
             alias=None,
             app_uuid=None,
             recipient=None,
             limit=None,
             offset=None):
        return self.client.request(
            'GET', ('Verify', 'Session', ),
            to_param_dict(self.list, locals()),
            response_type=ListSessionResponseObject,
            objects_type=Session)

    @validate_args(
        otp=[optional(of_type(six.text_type))],
        session_uuid=[of_type(six.text_type)]
    )
    def validate(self, session_uuid, otp=None):
        return self.client.request('POST', ('Verify', 'Session', session_uuid),
                                   to_param_dict(self.validate, locals()))




from plivo.utils.validators import *
from plivo.xml import (
    PlivoXMLElement,
    map_type
)


class StreamElement(PlivoXMLElement):
    _name = 'Stream'
    _nestable = [
    ]

    def __init__(
            self,
            content,
            bidirectional=None,
            audioTrack=None,
            streamTimeout=None,
            statusCallbackUrl=None,
            statusCallbackMethod=None,
            contentType=None,
            extraHeaders=None
    ):
        super(StreamElement, self).__init__()

        self.content = content
        self.bidirectional = bidirectional
        self.audioTrack = audioTrack
        self.streamTimeout = streamTimeout
        self.statusCallbackUrl = statusCallbackUrl
        self.statusCallbackMethod = statusCallbackMethod
        self.contentType = contentType
        self.extraHeaders = extraHeaders

    def to_dict(self):
        d = {
            'bidirectional': self.bidirectional,
            'audioTrack': self.audioTrack,
            'streamTimeout': self.streamTimeout,
            'statusCallbackUrl': self.statusCallbackUrl,
            'statusCallbackMethod': self.statusCallbackMethod,
            'contentType': self.contentType,
            'extraHeaders': self.extraHeaders,
        }
        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }


from ..base import PlivoResourceInterface
from requests import Request
from plivo.exceptions import ValidationError


FEEDBACK_API_PATH = "v1/Call/{}/Feedback/"


class CallFeedback(PlivoResourceInterface):
    def create(self, call_uuid, rating, issues=[], notes=""):
        if len(call_uuid) == 0:
            raise ValidationError('call_uuid cannot be empty')
        if not rating:
            raise ValidationError('rating cannot be empty')
        request_path = FEEDBACK_API_PATH.format(call_uuid)
        params_dict = {}
        params_dict['rating'] = rating
        if len(issues) > 0:
            params_dict['issues'] = issues
        if len(notes) > 0:
            params_dict['notes'] = notes
        params_dict['is_callinsights_request'] = True
        params_dict['callinsights_request_path'] = request_path
        return self.client.request('POST', ('Call', ), params_dict)



import string

from plivo.base import (PlivoResourceInterface)
from plivo.utils.validators import *


class Token(PlivoResourceInterface):
    @validate_args(
        iss=[required(of_type(six.text_type))],
        sub=[optional(of_type(six.text_type))],
        nbf=[optional(of_type(six.text_type))],
        exp=[optional(of_type(six.text_type))],
        incoming_allow=[optional(of_type(bool, string))],
        outgoing_allow=[optional(of_type(bool, string))],
        app=[optional(of_type(six.text_type))]
    )
    def create(self, iss, sub=None, nbf=None, exp=None, incoming_allow=None, outgoing_allow=None, app=None):
        if incoming_allow is True and sub is None:
            raise ValueError('sub is required when incoming_allow is true')
        else:
            params = {'iss': iss}

            if sub:
                params['sub'] = sub
            if nbf:
                params['nbf'] = nbf
            if exp:
                params['exp'] = exp
            if incoming_allow or outgoing_allow:
                params['per'] = {}
                params['per']['voice'] = {}
                if incoming_allow:
                    params['per']['voice']['incoming_allow'] = incoming_allow
                if outgoing_allow:
                    params['per']['voice']['outgoing_allow'] = outgoing_allow
            if app:
                params['app'] = app

            return self.client.request('POST', ('JWT', 'Token',), params, is_voice_request=True)



from plivo.utils.validators import validate_args, required, of_type_exact

class Location:
    @validate_args(
        latitude=[required(of_type_exact(str))],
        longitude=[required(of_type_exact(str))],
        name=[required(of_type_exact(str))],
        address=[required(of_type_exact(str))]
    )
    def __init__(self, latitude, longitude, name, address):
        self.latitude = latitude
        self.longitude = longitude
        self.name = name
        self.address = address



from plivo.utils.validators import *
from plivo.xml import PlivoXMLElement, map_type


class SayAsElement(PlivoXMLElement):
    _name = 'say-as'
    _nestable = []

    @property
    def interpret_as(self):
        return self.__interpret_as

    @interpret_as.setter
    def interpret_as(self, value):
        self.__interpret_as = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_interpret_as(self, value):
        self.interpret_as = value
        return self

    @property
    def format(self):
        return self.__format

    @format.setter
    def format(self, value):
        self.__format = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_format(self, value):
        self.format = value
        return self

    def __init__(
        self,
        content,
        interpret_as=None,
        format=None,
    ):

        super(SayAsElement, self).__init__()
        self.content = content
        self.interpret_as = interpret_as
        self.format = format

    def to_dict(self):
        d = {
            'interpret-as': self.interpret_as,
            'format': self.format,
        }
        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }



from plivo.utils.validators import *
from plivo.xml import (
    PlivoXMLElement,
    map_type,
    NumberElement,
    UserElement
)


class DialElement(PlivoXMLElement):
    _name = 'Dial'
    _nestable = ['User', 'Number']

    @property
    def action(self):
        return self.__action

    @action.setter
    def action(self, value):
        self.__action = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_action(self, value):
        self.action = value
        return self

    @property
    def method(self):
        return self.__method

    @method.setter
    def method(self, value):
        self.__method = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_method(self, value):
        self.method = value
        return self

    @property
    def hangup_on_star(self):
        return self.__hangup_on_star

    @hangup_on_star.setter
    def hangup_on_star(self, value):
        self.__hangup_on_star = bool(value) if value is not None else None

    @validate_args(
        value=[of_type_exact(bool)],
    )
    def set_hangup_on_star(self, value):
        self.hangup_on_star = value
        return self

    @property
    def time_limit(self):
        return self.__time_limit

    @time_limit.setter
    def time_limit(self, value):
        self.__time_limit = int(value) if value is not None else None

    @validate_args(
        value=[of_type(*six.integer_types)],
    )
    def set_time_limit(self, value):
        self.time_limit = value
        return self

    @property
    def timeout(self):
        return self.__timeout

    @timeout.setter
    def timeout(self, value):
        self.__timeout = int(value) if value is not None else None

    @validate_args(
        value=[of_type(*six.integer_types)],
    )
    def set_timeout(self, value):
        self.timeout = value
        return self

    @property
    def caller_id(self):
        return self.__caller_id

    @caller_id.setter
    def caller_id(self, value):
        self.__caller_id = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_caller_id(self, value):
        self.caller_id = value
        return self

    @property
    def caller_name(self):
        return self.__caller_name

    @caller_name.setter
    def caller_name(self, value):
        self.__caller_name = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_caller_name(self, value):
        self.caller_name = value
        return self

    @property
    def confirm_sound(self):
        return self.__confirm_sound

    @confirm_sound.setter
    def confirm_sound(self, value):
        self.__confirm_sound = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_confirm_sound(self, value):
        self.confirm_sound = value
        return self

    @property
    def confirm_timeout(self):
        return self.confirm_timeout

    @confirm_timeout.setter
    def confirm_timeout(self, value):
        self.confirm_timeout = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def confirm_timeout(self, value):
        self.confirm_timeout = value
        return self

    @property
    def confirm_key(self):
        return self.__confirm_key

    @confirm_key.setter
    def confirm_key(self, value):
        self.__confirm_key = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_confirm_key(self, value):
        self.confirm_key = value
        return self

    @property
    def dial_music(self):
        return self.__dial_music

    @dial_music.setter
    def dial_music(self, value):
        self.__dial_music = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_dial_music(self, value):
        self.dial_music = value
        return self

    @property
    def callback_url(self):
        return self.__callback_url

    @callback_url.setter
    def callback_url(self, value):
        self.__callback_url = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_callback_url(self, value):
        self.callback_url = value
        return self

    @property
    def callback_method(self):
        return self.__callback_method

    @callback_method.setter
    def callback_method(self, value):
        self.__callback_method = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_callback_method(self, value):
        self.callback_method = value
        return self

    @property
    def redirect(self):
        return self.__redirect

    @redirect.setter
    def redirect(self, value):
        self.__redirect = bool(value) if value is not None else None

    @validate_args(
        value=[of_type_exact(bool)],
    )
    def set_redirect(self, value):
        self.redirect = value
        return self

    @property
    def digits_match(self):
        return self.__digits_match

    @digits_match.setter
    def digits_match(self, value):
        self.__digits_match = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_digits_match(self, value):
        self.digits_match = value
        return self

    @property
    def digits_match_b_leg(self):
        return self.__digits_match_b_leg

    @digits_match_b_leg.setter
    def digits_match_b_leg(self, value):
        self.__digits_match_b_leg = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_digits_match_b_leg(self, value):
        self.digits_match_b_leg = value
        return self

    @property
    def sip_headers(self):
        return self.__sip_headers

    @sip_headers.setter
    def sip_headers(self, value):
        self.__sip_headers = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_sip_headers(self, value):
        self.sip_headers = value
        return self

    def __init__(
            self,
            action=None,
            method=None,
            hangup_on_star=None,
            time_limit=None,
            timeout=None,
            caller_id=None,
            caller_name=None,
            confirm_sound=None,
            confirm_timeout=None,
            confirm_key=None,
            dial_music=None,
            callback_url=None,
            callback_method=None,
            redirect=None,
            digits_match=None,
            digits_match_b_leg=None,
            sip_headers=None,
    ):
        super(DialElement, self).__init__()

        self.action = action
        self.method = method
        self.hangup_on_star = hangup_on_star
        self.time_limit = time_limit
        self.timeout = timeout
        self.caller_id = caller_id
        self.caller_name = caller_name
        self.confirm_sound = confirm_sound
        self.confirm_timeout = confirm_timeout
        self.confirm_key = confirm_key
        self.dial_music = dial_music
        self.callback_url = callback_url
        self.callback_method = callback_method
        self.redirect = redirect
        self.digits_match = digits_match
        self.digits_match_b_leg = digits_match_b_leg
        self.sip_headers = sip_headers

    def to_dict(self):
        d = {
            'action': self.action,
            'method': self.method,
            'hangupOnStar': self.hangup_on_star,
            'timeLimit': self.time_limit,
            'timeout': self.timeout,
            'callerId': self.caller_id,
            'callerName': self.caller_name,
            'confirmSound': self.confirm_sound,
            'confirmTimeout': self.confirm_timeout,
            'confirmKey': self.confirm_key,
            'dialMusic': self.dial_music,
            'callbackUrl': self.callback_url,
            'callbackMethod': self.callback_method,
            'redirect': self.redirect,
            'digitsMatch': self.digits_match,
            'digitsMatchBLeg': self.digits_match_b_leg,
            'sipHeaders': self.sip_headers,
        }
        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
            }

    def add_user(
            self,
            content,
            send_digits=None,
            send_on_preanswer=None,
            sip_headers=None,
    ):
        self.add(
            UserElement(
                content=content,
                send_digits=send_digits,
                send_on_preanswer=send_on_preanswer,
                sip_headers=sip_headers,
            ))
        return self

    def add_number(
            self,
            content,
            send_digits=None,
            send_on_preanswer=None,
    ):
        self.add(
            NumberElement(
                content=content,
                send_digits=send_digits,
                send_on_preanswer=send_on_preanswer,
            ))
        return self
